use [AdventureWorks2012]
GO

/* Perform any necessary cleanup. */
IF EXISTS(SELECT * FROM sys.views WHERE name = 'vSalesOrderDetail_1')
BEGIN
	DROP VIEW vSalesOrderDetail_1;
END
GO

IF EXISTS(SELECT * FROM sys.views WHERE name = 'vEmail')
BEGIN
	DROP VIEW vEmail;
END
GO

/* Prep work */
SET STATISTICS IO ON;
SET NOCOUNT ON;
--Remember to turn on actual execution plans:  Ctrl-M
GO

/* Create a view which gets "important" fields */
CREATE VIEW vSalesOrderDetail_1 AS
SELECT
    soh.SalesOrderID,
    soh.OrderDate,
    soh.DueDate,
    soh.Status,
    st.Name as SalesTerritory,
    cc.CardType,
    cc.CardNumber,
    soh.SubTotal,
    soh.TotalDue,
    sod.SalesOrderDetailID,
    sod.OrderQty,
    sod.UnitPrice,
    sod.LineTotal
FROM
    Sales.SalesOrderHeader soh
    INNER JOIN Sales.SalesOrderDetail sod ON soh.SalesOrderID = sod.SalesOrderID
    INNER JOIN Sales.SalesTerritory st ON soh.TerritoryID = st.TerritoryID
    INNER JOIN Sales.CreditCard cc ON soh.CreditCardID = cc.CreditCardID;
GO

/* Now create a view which uses the first view to get
	pertinent details for sending out e-mails. */	
CREATE VIEW vEmail AS
SELECT
    v.SalesOrderID,
    v.SalesOrderDetailID,
    v.LineTotal,
    be.BusinessEntityID,
    p.FirstName,
    p.LastName,
    e.EmailAddress
FROM
    vSalesOrderDetail_1 v
    LEFT OUTER JOIN Sales.SalesOrderHeader soh ON v.SalesOrderID = soh.SalesOrderID
    LEFT OUTER JOIN Sales.SalesPerson sp ON soh.SalesPersonID = sp.BusinessEntityID
    LEFT OUTER JOIN Person.BusinessEntity be ON sp.BusinessEntityID = be.BusinessEntityID
    LEFT OUTER JOIN Person.Person p ON p.BusinessEntityID = be.BusinessEntityID
    LEFT OUTER JOIN Person.EmailAddress e ON e.BusinessEntityID = p.BusinessEntityID;
GO

/* This is an exmaple of our nested view in action. */	
SELECT
    v.SalesOrderID,
    soh.AccountNumber,
    v.SalesOrderDetailID,
    v.LineTotal,
    sp.CommissionPct,
    v.BusinessEntityID
FROM
    vEmail v
    INNER JOIN Sales.SalesPerson sp ON v.BusinessEntityID = sp.BusinessEntityID
    INNER JOIN Sales.SalesOrderHeader soh ON soh.SalesOrderID = v.SalesOrderID
WHERE
    v.SalesOrderID IN (43659, 43660, 43661);

/* In contrast, here is the optimal form for this query. */
SELECT
    soh.SalesOrderID,
    soh.AccountNumber,
    sod.SalesOrderDetailID,
    sod.LineTotal,
    sp.CommissionPct,
    sp.BusinessEntityID
FROM
    Sales.SalesOrderHeader soh
    LEFT OUTER JOIN Sales.SalesOrderDetail sod ON soh.SalesOrderID = sod.SalesOrderID
    LEFT OUTER JOIN Sales.SalesPerson sp ON soh.SalesPersonID = sp.BusinessEntityID
WHERE
    soh.SalesOrderID IN (43659, 43660, 43661);